@@ -27,6 +27,7 @@ from typing_extensions import (
     Never,
     NotRequired,
     ParamSpec,
+    TypeAliasType,
     TypedDict,
     TypeVarTuple,
     Unpack,
@@ -519,6 +520,37 @@ def test_generics_work_with_many_parametrized_base_models():
     del generics
 
 
+def test_generics_reused() -> None:
+    """https://github.com/pydantic/pydantic/issues/11747
+
+    To fix an issue with recursive generics, we introduced a change in 2.11 that would
+    skip caching the parameterized model under specific circumstances. The following setup
+    is an example of where this would happen. As a result, we ended up with two different `A[int]`
+    classes, although they were the same in practice.
+    When serializing, we check that the value instances are matching the type, but we ended up
+    with warnings as `isinstance(value, A[int])` fails.
+    The fix was reverted as a refactor (https://github.com/pydantic/pydantic/pull/11388) fixed
+    the underlying issue.
+    """
+
+    T = TypeVar('T')
+
+    class A(BaseModel, Generic[T]):
+        pass
+
+    class B(BaseModel, Generic[T]):
+        pass
+
+    AorB = TypeAliasType('AorB', Union[A[T], B[T]], type_params=(T,))
+
+    class Main(BaseModel, Generic[T]):
+        ls: list[AorB[T]] = []
+
+    m = Main[int]()
+    m.ls.append(A[int]())
+    m.model_dump_json(warnings='error')
+
+
 def test_generic_config():
     data_type = TypeVar('data_type')
 
@@ -1445,7 +1477,9 @@ def test_deep_generic_with_referenced_inner_generic():
 
     with pytest.raises(ValidationError):
         InnerModel[int](a=['s', {'a': 'wrong'}])
+
     assert InnerModel[int](a=['s', {'a': 1}]).a[1].a == 1
+    assert InnerModel[int].model_fields['a'].annotation == Optional[list[Union[ReferencedModel[int], str]]]
 
 
 def test_deep_generic_with_multiple_typevars():
@@ -1670,6 +1704,26 @@ def test_generic_recursive_models_parametrized() -> None:
     Model2[str].model_rebuild()
 
 
+def test_generic_recursive_models_parametrized_with_model() -> None:
+    """https://github.com/pydantic/pydantic/issues/11748"""
+
+    T = TypeVar('T')
+
+    class Base(BaseModel, Generic[T]):
+        t: T
+
+    class Other(BaseModel):
+        child: 'Optional[Base[Other]]'
+
+    with pytest.raises(ValidationError):
+        # In v2.0-2.10, this unexpectedly validated fine (The core schema of Base[Other].t was an empty model).
+        # Since v2.11, building `Other` raised an unhandled exception.
+        # Now, it works as expected.
+        Base[Other].model_validate({'t': {}})
+
+    Base[Other].model_validate({'t': {'child': {'t': {'child': None}}}})
+
+
 @pytest.mark.xfail(reason='Core schema generation is missing the M1 definition')
 def test_generic_recursive_models_inheritance() -> None:
     """https://github.com/pydantic/pydantic/issues/9969"""
